--------------------------------------------------------------------------------
-- [VHDL Praktikum SS2004]
--
-- Aufgabe 3:     Minima-Rechenwerk
-- Entity:        E_MINIMA (Top-Level-Entity)               
--
-- Andreas Schibilla (ii4900)
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;


entity E_ALU is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        C_IMPULS:    in  std_logic;
        
        A_IN:        in  std_logic;
        S_IN:        in  std_logic;
        C_IN:        in  std_logic;
        L_IN:        in  std_logic;
        R_IN:        in  std_logic;
        N_IN:        in  std_logic;
        
        BR_IN:       in  std_logic;
        RAM_IN:      in  std_logic;
        
        ACCU_S_OUT:  out std_logic;
        ACCU_Q_OUT:  out std_logic_vector(19 downto 0);
                     
        -- zum Debuggen
        DE_AS_OUT:   out std_logic;
        DE_SH_OUT:   out std_logic;
        DE_ADD_OUT:  out std_logic
    );
end E_ALU;


architecture A_ALU of E_ALU is
-- Komponenten:
component ACCU IS
	PORT
	(
		clock		: IN STD_LOGIC ;
		enable		: IN STD_LOGIC ;
		shiftin		: IN STD_LOGIC ;
		aclr		: IN STD_LOGIC ;
		q		: OUT STD_LOGIC_VECTOR (19 DOWNTO 0);
		shiftout		: OUT STD_LOGIC 
	);
END component;

component E_ADDER is    
    port (         
        CLK:      in  std_logic;
        RST:      in  std_logic;
        A_IMPULS: in  std_logic;
        B_IMPULS: in  std_logic;
        S_IN:     in  std_logic;
        BIT1_IN:  in  std_logic;
        BIT2_IN:  in  std_logic;
        S_OUT:    out std_logic
    );
end component;

component E_ADDSUB is    
    port (         
        CLK:      in  std_logic;
        A_IMPULS: in  std_logic;
        B_IMPULS: in  std_logic;
        C_IMPULS: in  std_logic;
        A_IN:     in  std_logic;
        S_IN:     in  std_logic;
        C_IN:     in  std_logic;
        RAM_IN:   in  std_logic;
        BR_IN:    in  std_logic;
        S_OUT:    out std_logic
    );
end component;

component E_SHIFTING is    
    port (
        CLK:         in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        L_IN:        in  std_logic;
        R_IN:        in  std_logic;
        N_IN:        in  std_logic;
        S_IN:        in  std_logic;
        S_OUT:       out std_logic
    );
end component;

signal INT_ADDSUB_OUT   : std_logic;
signal INT_ACCU_OUT     : std_logic;
signal INT_SHIFTING_OUT : std_logic;
signal INT_ADDER_OUT    : std_logic;
signal INT_IMPULS       : std_logic;

begin      
 -- Komponenten einbinden
 ADDSUB_inst : E_ADDSUB PORT MAP (     CLK         => CLK,
                                       A_IMPULS    => A_IMPULS,
                                       B_IMPULS    => B_IMPULS,
                                       C_IMPULS    => C_IMPULS,
                                       A_IN        => A_IN,
                                       S_IN        => S_IN,
                                       C_IN        => C_IN,
                                       RAM_IN      => RAM_IN,
                                       BR_IN       => BR_IN,
                                       S_OUT       => INT_ADDSUB_OUT
                                 );
                                 
 SHIFTING_inst : E_SHIFTING PORT MAP ( CLK         => CLK,
                                       A_IMPULS    => A_IMPULS,
                                       B_IMPULS    => B_IMPULS,
                                       L_IN        => L_IN,
                                       R_IN        => R_IN,
                                       N_IN        => N_IN,
                                       S_IN        => INT_ACCU_OUT,
                                       S_OUT       => INT_SHIFTING_OUT
                                     );
                                     
 ADDER_inst : E_ADDER PORT MAP (       CLK         => CLK,
                                       RST         => RST, 
                                       A_IMPULS    => A_IMPULS,
                                       B_IMPULS    => B_IMPULS,
                                       S_IN        => S_IN,
                                       BIT1_IN     => INT_ADDSUB_OUT,
                                       BIT2_IN     => INT_SHIFTING_OUT,
                                       S_OUT       => INT_ADDER_OUT
                               );
                               
 ACCU_inst : ACCU PORT MAP (		      clock       => CLK,
		                                 enable      => A_IMPULS or INT_IMPULS,
		                                 shiftin     => INT_ADDER_OUT,
		                                 aclr        => RST,
		                                 q           => ACCU_Q_OUT,
		                                 shiftout    => INT_ACCU_OUT
	                        );                               
	                        
 ACCU_S_OUT <= INT_ACCU_OUT;
 
 --zum Debuggen 
 DE_AS_OUT   <= INT_ADDSUB_OUT;
 DE_SH_OUT   <= INT_SHIFTING_OUT;
 DE_ADD_OUT  <= INT_ADDER_OUT;
 
 -- Steuerimpuls fr das Schieben des Accus vorbereiten
 P1: process(CLK, A_IMPULS)
 begin
   if rising_edge(CLK) then
      if A_IMPULS = '1' then
         INT_IMPULS <= '1';
      else
         INT_IMPULS <= '0';
      end if;
   end if;
 end process P1; 
             
end A_ALU;




