--------------------------------------------------------------------------------
-- [VHDL Praktikum SS2004]
--
-- Aufgabe 3:     von Neumann Addierwerk / Fibonacci-Zahlen-Generator
-- Entity:        E_RW (Rechenwerk)               
--
-- Andreas Schibilla (ii4900)
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;


entity E_SPEICHER is    
    port (
        CLK:         in  std_logic; 
        RST:         in  std_logic;
        ACCU_S_IN:   in  std_logic;
        BZ_S_IN:     in  std_logic;
        F:           in  std_logic;
        U:           in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        ADDR_BUS:    in  std_logic_vector(7 downto 0);
        
        S_OUT:       out std_logic;                      
        -- temp.
        Q_OUT        : OUT STD_LOGIC_VECTOR (19 DOWNTO 0)
    );
end E_SPEICHER;


architecture A_SPEICHER of E_SPEICHER is

type t_speicher is(IDLE, LADEN1, LADEN2, LADEN3, SCHIEBEN, SPEICHERN);
              
component RAM
   PORT
   (
      address     : IN STD_LOGIC_VECTOR (7 DOWNTO 0);
      we          : IN STD_LOGIC  := '1';
      inclock     : IN STD_LOGIC ;
      outclock    : IN STD_LOGIC ;
      data        : IN STD_LOGIC_VECTOR (19 DOWNTO 0);
      q           : OUT STD_LOGIC_VECTOR (19 DOWNTO 0)
   );
end component;              
              
component SHIFTREG
   PORT
   (
      clock       : IN STD_LOGIC ;
      enable      : IN STD_LOGIC ;
      shiftin     : IN STD_LOGIC ;
      load        : IN STD_LOGIC ;
      aclr        : IN STD_LOGIC ;
      data        : IN STD_LOGIC_VECTOR (19 DOWNTO 0);
      q           : OUT STD_LOGIC_VECTOR (19 DOWNTO 0);
      shiftout    : OUT STD_LOGIC 
   );
end component;

signal INT_ADDR_BUS:    std_logic_vector(7 downto 0);
signal INT_ADDR_F:      std_logic_vector(1 downto 0);
signal INT_SAVE_F:      std_logic;
signal INT_WE:          std_logic;
signal INT_INCLOCK:     std_logic;
signal INT_OUTCLOCK:    std_logic;
signal INT_DATA:        std_logic_vector(19 downto 0);
signal INT_Q:           std_logic_vector(19 downto 0);

signal INT_SH_ENABLE:   std_logic;
signal INT_SH_IN:       std_logic;
signal INT_SH_LOAD:     std_logic;
signal INT_SH_ACLR:     std_logic;
--signal INT_SH_DATA:     std_logic_vector(19 downto 0);
signal INT_SH_Q:        std_logic_vector(19 downto 0);

signal STATE:           t_speicher;
begin                    

RAM_inst : RAM PORT MAP (
      -- IN --
      address        => INT_ADDR_BUS,
      we             => INT_WE,
      inclock        => INT_INCLOCK,
      outclock       => INT_OUTCLOCK,
      data           => INT_DATA,
      -- OUT --
      q              => INT_Q
   );

SHIFTREG_inst : SHIFTREG PORT MAP (
      -- IN --
      clock          => CLK,
      enable         => INT_SH_ENABLE,
      shiftin        => INT_SH_IN,
      load           => INT_SH_LOAD,
      aclr           => INT_SH_ACLR,
      data           => INT_Q,
      -- OUT --
      q              => INT_SH_Q,
      shiftout       => S_OUT
   );


 INT_ADDR_F(0) <= F;
 INT_ADDR_F(1) <= INT_SAVE_F;
 
 with INT_ADDR_F select
 INT_ADDR_BUS  <= "00000101"     when "11",
                  ADDR_BUS       when "00",
                  ADDR_BUS       when "01",
                  ADDR_BUS       when "10";


 
 with F select
 INT_SH_IN     <= BZ_S_IN        when '1',
                  ACCU_S_IN      when '0';
      
 INT_DATA      <= INT_SH_Q;

      
 -- temp.      
 Q_OUT <= INT_SH_Q;
 

                                    
 P1: process(RST,CLK)
 begin
    if RST = '1' then                  -- Reset-Signal liegt an                  
    
      INT_WE         <= '0';
      INT_INCLOCK    <= '0';
      INT_OUTCLOCK   <= '0';
      --INT_DATA       <= (others => '0');
                    
      INT_SH_ENABLE  <= '0';
      INT_SH_LOAD    <= '0';
      INT_SH_ACLR    <= '0';     
     
      INT_SAVE_F     <= '0';
      STATE          <= IDLE;
                    
       
    elsif rising_edge(CLK) then
             
       case STATE is             
            when IDLE         =>  -- Alle Signale initialsieren
                                 INT_SAVE_F        <= '0';
                                 INT_INCLOCK       <= '0';
                                 INT_OUTCLOCK      <= '0';
                                 INT_SH_LOAD       <= '0';
                                 INT_SH_ENABLE     <= '0';
                                 INT_WE            <= '0';
                                 if B_IMPULS = '1' then
                                    STATE          <= LADEN1;
                                 end if;
                                 
            when LADEN1       => INT_INCLOCK       <= '1';     -- jetzt Adresse festlegen
                                 STATE             <= LADEN2;
            
            when LADEN2       => INT_INCLOCK       <= '0';
                                 INT_OUTCLOCK      <= '1';     -- jetzt Wort aus Speicher laden
                                 INT_SH_LOAD       <= '1';     -- Wert bei nchstem Takt in SHIFTREG speichern
                                 INT_SH_ENABLE     <= '1';     -- Enable fr LOAD
                                 STATE             <= LADEN3;
                                 
            when LADEN3       => INT_SH_LOAD       <= '0';     
                                 INT_INCLOCK       <= '0';
                                 INT_OUTCLOCK      <= '0';
                                 INT_SH_ENABLE     <= '1';     -- Nach Ladevorgang das Schieben starten
                                 INT_SAVE_F        <= '1';
                                 STATE             <= SCHIEBEN;
                           
            when SCHIEBEN     => INT_WE            <= '1';
                                 if A_IMPULS = '1' then        -- wenn A-Impuls dann: 
                                    INT_INCLOCK    <= '0';
                                    INT_SH_ENABLE  <= '1';     -- REINSCHIEBEN (letztes BIT geht raus)
                                    STATE          <= SCHIEBEN;
                                 else                          -- ansonsten nicht mehr Schieben
                                    INT_SH_ENABLE  <= '0';
                                    if (F='1') or (U='1') then    -- wenn F oder U
                                       INT_INCLOCK    <= '1';     -- dann Wort im Speicher ablegen
                                       
                                    end if;
                                    STATE          <= SPEICHERN;
                                 end if;
                           
            when SPEICHERN    => INT_INCLOCK       <= '0';     -- Speichervorgang beenden
                                 INT_WE            <= '0';
                                 INT_SAVE_F        <= '0';
                                 STATE             <= IDLE;
       end case;      
             
             
       
    end if;
    
 end process P1;

end A_SPEICHER;




