--------------------------------------------------------------------------------
-- [VHDL Praktikum SS2004]
--
-- Aufgabe 3:     Minima-Rechenwerk
-- Entity:        E_MINIMA (Top-Level-Entity)               
--
-- Andreas Schibilla (ii4900)
--------------------------------------------------------------------------------
library ieee;
use ieee.std_logic_1164.all;


entity E_STEUERWERK is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        A_IMPULS:    in  std_logic;
        B_IMPULS:    in  std_logic;
        ACCUBIT_IN:  in  std_logic;
        RAM_IN:      in  std_logic;
        
        E_OUT:       out std_logic;
        F_OUT:       out std_logic;
        P_OUT:       out std_logic;
        Q_OUT:       out std_logic;
        N_OUT:       out std_logic;
        R_OUT:       out std_logic;
        L_OUT:       out std_logic;
        C_OUT:       out std_logic;
        A_OUT:       out std_logic;
        S_OUT:       out std_logic;
        U_OUT:       out std_logic;
        H_OUT:       out std_logic;
        ADDR_OUT:    out std_logic_vector (7 downto 0);
        
        BZ_OUT:      out std_logic;
        BR_OUT:      out std_logic
    );
end E_STEUERWERK;


architecture A_STEUERWERK of E_STEUERWERK is

 -- Komponente fr Befehlszhler:
component BZ
	PORT
	(
		clock		   : IN STD_LOGIC ;
		enable		: IN STD_LOGIC ;
		shiftin		: IN STD_LOGIC ;
		shiftout		: OUT STD_LOGIC 
	);
end component;

 -- Komponente fr Befehlsregister:
component E_BR is    
    port (
        CLK:      in  std_logic; 
        RST:      in  std_logic;
        BZ_IN:    in  std_logic;
        RAM_IN:   in  std_logic;
        E_IN:     in  std_logic;
        A_IMPULS: in  std_logic;
        S_OUT:    out std_logic;
        D_OUT:    out STD_LOGIC_VECTOR (19 DOWNTO 0)
    );
end component;

 -- Komponente fr Plus 1 Addierer:
component E_PLUS_EINS is    
    port (         
        CLK:      in std_logic;
        A_IMPULS: in  std_logic;
        S_IN:     in  std_logic;
        S_OUT:    out std_logic
    );
end component;

 -- Komponente fr Steuerregister:
component E_SR is    
    port (
        CLK:         in  std_logic;
        RST:         in  std_logic;
        B_IMPULS:    in  std_logic;
        ACCUBIT_IN:  in  std_logic;
        REG_IN:      in  std_logic_vector(19 downto 0);
        E_OUT:       out std_logic;
        F_OUT:       out std_logic;
        P_OUT:       out std_logic;
        Q_OUT:       out std_logic;
        N_OUT:       out std_logic;
        R_OUT:       out std_logic;
        L_OUT:       out std_logic;
        C_OUT:       out std_logic;
        A_OUT:       out std_logic;
        S_OUT:       out std_logic;
        U_OUT:       out std_logic;
        H_OUT:       out std_logic;
        ADDR_OUT:    out std_logic_vector (7 downto 0)
    );
end component; 

 -- Koppelsignale
signal INT_BZ_IN  : std_logic;
signal INT_BZ_OUT : std_logic;

signal INT_BR_S_OUT : std_logic;
signal INT_BR_D_OUT : std_logic_vector(19 downto 0);

signal INT_E      : std_logic;

begin      

 -- PORT-MAP fr Befehlszhler:
 BZ_inst : BZ PORT MAP (
		clock	         => CLK,
		enable	      => A_IMPULS,
		shiftin	      => INT_BZ_IN,
		shiftout	      => INT_BZ_OUT
	);

 -- PORT-MAP fr Befehlsregister:
 E_BR_inst : E_BR PORT MAP (
        CLK          => CLK,
        RST          => RST,
        BZ_IN        => INT_BZ_OUT,
        RAM_IN       => RAM_IN,
        E_IN         => INT_E,
        A_IMPULS     => A_IMPULS,
        S_OUT        => INT_BR_S_OUT,
        D_OUT        => INT_BR_D_OUT
    );

 -- PORT-MAP fr Plus 1 Addierer:
 E_PLUS_EINS_inst : E_PLUS_EINS PORT MAP (
        CLK          => CLK,
        A_IMPULS     => A_IMPULS,
        S_IN         => INT_BR_S_OUT,
        S_OUT        => INT_BZ_IN
    );

 -- PORT-MAP fr Steuerregister:
 -- Komponente fr Steuerregister:
 E_SR_inst : E_SR PORT MAP (
        CLK          => CLK,
        RST          => RST,
        B_IMPULS     => B_IMPULS,
        ACCUBIT_IN   => ACCUBIT_IN,
        REG_IN       => INT_BR_D_OUT,
        E_OUT        => INT_E,
        F_OUT        => F_OUT,
        P_OUT        => P_OUT,
        Q_OUT        => Q_OUT,
        N_OUT        => N_OUT,
        R_OUT        => R_OUT,
        L_OUT        => L_OUT,
        C_OUT        => C_OUT,
        A_OUT        => A_OUT,
        S_OUT        => S_OUT,
        U_OUT        => U_OUT,
        H_OUT        => H_OUT,
        ADDR_OUT     => ADDR_OUT
    );


 -- Steuerwerk-Ausgnge zuordnen:
E_OUT    <= INT_E; 
BZ_OUT   <= INT_BZ_OUT;
BR_OUT   <= INT_BR_S_OUT;
  
end A_STEUERWERK;




